# Define the kmod package name here.
%define kmod_name fmc

%define kmods_built %{kmod_name} %{kmod_name}-chardev %{kmod_name}-fakedev %{kmod_name}-trivial %{kmod_name}-write-eeprom

%define tagversion 2017-06
%define packageversion %(echo %{tagversion} | sed 's/-/./g')


# If kversion isn't defined on the rpmbuild line, define it here.
%{!?kversion: %define kversion 3.10.0-693.el7.%{_target_cpu}}

Name:    %{kmod_name}-kmod
Version: 0.1
Release: %{packageversion}.2%{?dist}
Group:   System Environment/Kernel
License: GPLv2
Summary: %{kmod_name} kernel module(s)
URL:     http://www.kernel.org/

BuildRequires: perl emacs sed gawk git hardlink texinfo-tex
BuildRequires: redhat-rpm-config
ExclusiveArch: x86_64

# Sources.
Source0:  %{kmod_name}-%{version}.tar.bz2
Source5:  GPL-v2.0.txt
Source10: kmodtool-%{kmod_name}-el7.sh

Source50: %{kmod_name}-tools-%{version}.tar.bz2
Patch1: fmc-out-of-tree.patch
Patch2: fmc-tools-pythonpath.patch

# Magic hidden here.
%{expand:%(sh %{SOURCE10} rpmtemplate %{kmod_name} %{kversion} "")}

# Disable the building of the debug package(s).
%define debug_package %{nil}

%description
This package provides the %{kmod_name} kernel module(s).
It is built to depend upon the specific ABI provided by a range of releases
of the same variant of the Linux kernel and not on any one specific build.

%package headers
Summary:	header files for %{kmod_name}-%{version}

%description headers
Provides the %{kmod_name} kernel headers.

%package tools
Requires:	%{kmod_name}-kmod
Summary:        tools for %{kmod_name}-%{version}

%description tools
Provides the %{kmod_name} tools.

%prep
%setup -q -n %{kmod_name}-%{version}

for thiskmod in %{kmods_built}; do
    echo "override ${thiskmod} * weak-updates/%{kmod_name}" >> kmod-%{kmod_name}.conf
done

%patch1 -p1 -b .outoftree

%{__mkdir_p} tools
pushd tools
tar xvf %{SOURCE50}
cd %{kmod_name}-tools-%{version}
GIT_VERSION=$(git describe --dirty --long --tags)
%patch2 -p1
popd

cat << EOF > fmc-version-env
GIT_VERSION=${GIT_VERSION}
FMC_BUS_VERSION=${GIT_VERSION}
EOF



%build
KSRC=%{_usrsrc}/kernels/%{kversion}
%{__make} -C "${KSRC}" %{?_smp_mflags} modules M=$PWD

# Cleanup the mappings
for line in $(cat modules.order); do
  kmod=$(basename ${line})
  echo "kernel/extra/${kmod}" >> fmc.order
done

awk '{print $1 "\t" $2 "\textra/fmc\t" $4}' Module.symvers > fmc.symvers


pushd tools/%{kmod_name}-tools-%{version}

cd doc
%{__make} %{?_smp_mflags}
cd ..

cd sdb-lib
# Fix up for downstream usage
sed -i -e 's|<sdb.h>|<linux/sdb.h>|' libsdbfs.h

# Build for linkage into kernel
%{__make} %{?_smp_mflags} EXTRACFLAGS='-I../kernel/include -mcmodel=kernel'
mv glue.o glue.o.kmod
mv access.o access.o.kmod

# Build for linkage into userspace
%{__make} %{?_smp_mflags} EXTRACFLAGS='-I../kernel/include'
cd ..

cd tools
%{__make} %{?_smp_mflags}
cd ..

popd


%install
%{__rm} -rf %{buildroot}

%{__install} -d %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/


for thiskmod in %{kmods_built}; do
    %{__install} ${thiskmod}.ko %{buildroot}/lib/modules/%{kversion}/extra/%{kmod_name}/
done

%{__install} -d %{buildroot}%{_sysconfdir}/depmod.d/
%{__install} kmod-%{kmod_name}.conf %{buildroot}%{_sysconfdir}/depmod.d/

%{__install} -d %{buildroot}%{_defaultdocdir}/kmod-%{kmod_name}-%{version}/
%{__install} %{SOURCE5} %{buildroot}%{_defaultdocdir}/kmod-%{kmod_name}-%{version}/
%{__install} fmc.order %{buildroot}%{_defaultdocdir}/kmod-%{kmod_name}-%{version}/modules.order
%{__install} fmc.symvers %{buildroot}%{_defaultdocdir}/kmod-%{kmod_name}-%{version}/Module.symvers

# strip the modules(s)
find %{buildroot} -type f -name \*.ko -exec %{__strip} --strip-debug \{\} \;

# Sign the modules(s)
%if %{?_with_modsign:1}%{!?_with_modsign:0}
# If the module signing keys are not defined, define them here.
%{!?privkey: %define privkey %{_sysconfdir}/pki/SECURE-BOOT-KEY.priv}
%{!?pubkey: %define pubkey %{_sysconfdir}/pki/SECURE-BOOT-KEY.der}
for module in $(find %{buildroot} -type f -name \*.ko);
do %{__perl} /usr/src/kernels/%{kversion}/scripts/sign-file \
sha256 %{privkey} %{pubkey} $module;
done
%endif

# Create kmod headers
%{__install} -d %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/linux
%{__cp} *.h %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/linux

%{__install} -d %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/sdb-lib
%{__cp} tools/%{kmod_name}-tools-%{version}/sdb-lib/*.h %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/sdb-lib

%{__install} sdb.h %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/sdb-lib

echo '%{tagversion}' > %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/version
%{__install} fmc-version-env %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/fmc-version-env
%{__install} fmc.order %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/modules.order
%{__install} fmc.symvers %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/Module.symvers
%{__install} tools/%{kmod_name}-tools-%{version}/sdb-lib/access.o.kmod %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/access.o
%{__install} tools/%{kmod_name}-tools-%{version}/sdb-lib/glue.o.kmod %{buildroot}%{_usrsrc}/kmods/%{kmod_name}-%{version}/glue.o

# install tools
%{__install} -d %{buildroot}%{_sbindir}/
%{__install} -d %{buildroot}%{_libexecdir}/fmc/
%{__install} tools/%{kmod_name}-tools-%{version}/tools/fmc-mem %{buildroot}%{_sbindir}
%{__install} tools/%{kmod_name}-tools-%{version}/tools/fru-dump %{buildroot}%{_libexecdir}/fmc
%{__install} tools/%{kmod_name}-tools-%{version}/tools/fru-generator %{buildroot}%{_libexecdir}/fmc
%{__install} -d %{buildroot}%{_libdir}/
%{__install} tools/%{kmod_name}-tools-%{version}/sdb-lib/libsdbfs.a %{buildroot}%{_libdir}/

%{__install} -d %{buildroot}%{_libdir}/libsdbfs/
%{__install} tools/%{kmod_name}-tools-%{version}/sdb-lib/access.o %{buildroot}%{_libdir}/libsdbfs
%{__install} tools/%{kmod_name}-tools-%{version}/sdb-lib/glue.o %{buildroot}%{_libdir}/libsdbfs

#  I feel like libipmi will get confused with real ipmi libs
#  so I'm shoving it into /usr/libexec/ to prevent confusion
%{__install} -d %{buildroot}%{_libexecdir}/%{kmod_name}/libipmi
%{__install} tools/%{kmod_name}-tools-%{version}/tools/libipmi/__init__.py %{buildroot}%{_libexecdir}/%{kmod_name}/libipmi/
%{__install} tools/%{kmod_name}-tools-%{version}/tools/libipmi/fmc_eeprom.py %{buildroot}%{_libexecdir}/%{kmod_name}/libipmi/
%{__install} tools/%{kmod_name}-tools-%{version}/tools/libipmi/libipmi.so %{buildroot}%{_libexecdir}/%{kmod_name}/libipmi/

# Install docs
%{__install} tools/%{kmod_name}-tools-%{version}/doc/fmc-bus.txt %{buildroot}%{_defaultdocdir}/kmod-%{kmod_name}-%{version}/

# Install infopage
%{__install} -d %{buildroot}%{_infodir}
%{__install} tools/%{kmod_name}-tools-%{version}/doc/fmc-bus.info %{buildroot}%{_infodir}/

hardlink -c -vv %{buildroot}/*


%clean
%{__rm} -rf %{buildroot}

%files headers
%defattr(0644,root,root,0755)
%{_usrsrc}/kmods/%{kmod_name}-%{version}

%files tools
%defattr(0644,root,root,0755)
# management tools either make use of dev nodes with
#  group dialout, or outright calls to the kernel
%attr(4750,root,dialout) %{_sbindir}/*
%attr(0755,root,root) %{_libexecdir}/%{kmod_name}/*
%{_libdir}/*


%changelog
* Fri Sep 01 2017 Akemi Yagi<toracat@elrepo.org> - 0.1-2017.06.2
- provide objects in the right format [elrepo bug #668]

* Mon Aug 28 2017 Akemi Yagi<toracat@elrepo.org> - 0.1-2017.06.1
- rebase FMC off of upstream kernel
- submitted by jcpunk [elrepo bug #775]

* Tue Jan 10 2017 Akemi Yagi<toracat@elrepo.org> - 0.0-2
- Add a headers package [elrepo bug #709]

* Fri Nov 04 2016 Akemi Yagi<toracat@elrepo.org> - 0.0-1
- Initial el7 build of the kmod package
- Source code from kernel version 4.9-rc3
